package com.xcheng.scannere3;

import android.app.Notification;
import android.app.NotificationManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.SystemProperties;
import android.util.Log;
import java.lang.Thread;
import java.lang.Exception;
import java.util.Timer;
import java.util.TimerTask;

//add by yangzhichao for t0581 20200915 start
import com.xcheng.scannere3.codesetting.BaseSetting;
import android.view.KeyEvent;
//add by yangzhichao for t0581 20200915 end
public class ScanTestReceiver extends BroadcastReceiver{

    private static final String TAG = "ScanTestReceiver";
    private static final int SCAN_NOTIFICATION_ID = 10010;
    private NotificationManager mNm = null;

    private static final String ACTION_OPEN_SCAN_BROADCAST = ScanUtil.ACTION_OPEN_SCAN_BROADCAST;
    private static final String ACTION_CLOSE_SCAN_BROADCAST = ScanUtil.ACTION_CLOSE_SCAN_BROADCAST;

    //add by yangzhichao for t0581 20200915 start
    private static final String SCANKEY = ScanUtil.SCANKEY;
    //add by yangzhichao for t0581 20200915 end

    private static final String ACTION_ENABLE_TYPE = ScanUtil.ACTION_ENABLE_TYPE;
    private static final String ACTION_DISABLE_TYPE = ScanUtil.ACTION_DISABLE_TYPE;
    private static final String SCANTYPE = ScanUtil.SCANTYPE;

    @Override
    public void onReceive(Context context, Intent intent) {
        // TODO Auto-generated method stub
        String action = intent.getAction();
        int scanKey = intent.getIntExtra(SCANKEY, -1);
        String symbology = intent.getStringExtra(SCANTYPE);
        mNm = (NotificationManager)context.getSystemService(Context.NOTIFICATION_SERVICE);

//        Log.d(TAG, "onReceive: action = " + action + " symbology = " + symbology);

        if (ACTION_OPEN_SCAN_BROADCAST.equals(action) && isScanKeyEnable(context,scanKey)) {
            if (ScanTestService.isScanServiceRunning()) {
//                Log.d(TAG, "ScanTestService running");

                if (ScanTestService.isInScanActivity()) {
                    Log.d(TAG, "In ScanTestActivity, ignore ACTION_OPEN_SCAN_BROADCAST");
                } else if (!E3Util.IsDecoding() && ScanUtil.isActivityAllowScan(context)) {
                    E3Util.doDecode();
                }
            } else {
//                Log.d(TAG, "ScanTestService startService");
                context.startService(new Intent(context, ScanTestService.class));

                final Timer delayTmDecode = new Timer();
                delayTmDecode.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        if (ScanTestService.isScanServiceRunning()) {
                            Log.d(TAG, "doDecode after start ScanTestService");

                            if (ScanTestService.isInScanActivity()) {
                                Log.d(TAG, "In ScanTestActivity, ignore ACTION_OPEN_SCAN_BROADCAST");
                            } else if (!E3Util.IsDecoding() && ScanUtil.isActivityAllowScan(context)) {
                                E3Util.doDecode();
                            }
                            delayTmDecode.cancel();
                        }
                    }
                }, 100, 20);
            }

            if (isNotificationEnable(context)){
                sendScanNotification(context);
            }
        } else if (ACTION_CLOSE_SCAN_BROADCAST.equals(action)) {
            if (ScanTestService.isInScanActivity()) {
                Log.d(TAG, "In ScanTestActivity, ignore ACTION_CLOSE_SCAN_BROADCAST");
            } else if (E3Util.IsDecoding() && ScanUtil.isActivityAllowScan(context)) {
                E3Util.stopDecode();
            }

            if (isNotificationEnable(context)){
                mNm.cancel(SCAN_NOTIFICATION_ID);
            }
        } else if (ACTION_ENABLE_TYPE.equals(action) && isValidType(context, symbology)) {
            E3Util.setCodeTypeOnOff(symbology, true);
        } else if (ACTION_DISABLE_TYPE.equals(action) && isValidType(context, symbology)) {
            E3Util.setCodeTypeOnOff(symbology, false);
        }
        //add by yangzhichao for t0581 20200915 start
        else if (ScanUtil.ACTION_CONTROL_SCANKEY.equals(action)) {
            Log.d(TAG, "ACTION_CONTROL_SCANKEY");
            int scanKeyCode = intent.getIntExtra(ScanUtil.EXTRA_SCANKEY_CODE, -1);
            boolean scanKeyEnable = intent.getBooleanExtra(ScanUtil.EXTRA_SCANKEY_STATUS, true);
            SetScanKeyEnable(context, scanKeyCode, scanKeyEnable);
        }
        //add by yangzhichao for t0581 20200915 end
        else if (ScanUtil.ACTION_CONTROL_DATA_EVENT.equals(action)) {
            String dataEvent = intent.getStringExtra(ScanUtil.EXTRA_DATA_EVENT);
            setDataEvent(context, dataEvent);
        } else if (ScanUtil.ACTION_LOAD_SETTINGS.equals(action)) {
            ScanUtil.loadSettings(context);
        } else if (ScanUtil.ACTION_SAVE_SETTINGS.equals(action)) {
            ScanUtil.saveSettings(context);
        } else if (ScanUtil.ACTION_SCANNER_ENGINE.equals(action)) {
            String engine = intent.getStringExtra(ScanUtil.EXTRA_SCANNER_ENGINE);
            setScannerEngine(context, engine);
        } else if (ScanUtil.ACTION_START_SERVICE.equals(action)) {
            if (ScanTestService.isScanServiceRunning()) {
                Log.d(TAG, "ACTION_START_SERVICE: ScanTestService already running");
            } else {
                Log.d(TAG, "ACTION_START_SERVICE: ScanTestService startService");
                
                //E3Util.releaseDecodeLibrary();
                context.startService(new Intent(context, ScanTestService.class));
            }
        } else if (ScanUtil.ACTION_STOP_SERVICE.equals(action)) {
            if (ScanTestService.isScanServiceRunning()) {
                Log.d(TAG, "ACTION_STOP_SERVICE: ScanTestService is running");
                
                //E3Util.releaseDecodeLibrary();
                context.stopService(new Intent(context, ScanTestService.class));
            }
        } else if (Intent.ACTION_BOOT_COMPLETED.equals(action)) {
            if (ScanTestService.isScanServiceRunning()) {
                Log.d(TAG, "ACTION_BOOT_COMPLETED: ScanTestService already running");
            } else {
                Log.d(TAG, "ACTION_BOOT_COMPLETED: ScanTestService startService");
                context.startService(new Intent(context, ScanTestService.class));
            }
        }
    }

    private  boolean isValidType(Context context, String type) {
        for (int i = 0; i < ScanUtil.MAX_TYPE; i++) {
            if (context.getString(ScanUtil.SP_TITLE_RES[i]).equals(type)) {
                return true;
            }
        }
        return false;
    }

    private void setDataEvent(Context context, String dataEvent) {
        SharedPreferences sharedPreferences = ScanUtil.getSharedPreferences(context);
        Editor editor = sharedPreferences.edit();
        editor.putString(ScanUtil.DATA_RECEIVE_KEY, dataEvent);
        editor.commit();
    }

    private void setScannerEngine(Context context, String engine) {
        Log.d(TAG, "setScannerEngine: engine = " + engine);
        SharedPreferences sharedPreferences = ScanUtil.getSharedPreferences(context);
        Editor editor = sharedPreferences.edit();
        editor.putString(ScanUtil.EXTRA_SCANNER_ENGINE, engine);
        editor.commit();
    }

    private boolean isNotificationEnable(Context context) {
        boolean enable = ScanUtil.getSharedPreferences(context).getBoolean(
                ScanSettingActivity.APP_NOTIFICATION_KEY, true);
        return enable;
    }

    private void sendScanNotification(Context context) {
        Notification notify = new Notification.Builder(context)
            .setSmallIcon(R.mipmap.ic_launcher)
            .build();
        mNm.notify(SCAN_NOTIFICATION_ID, notify);
    }

    //add by yangzhichao for t0581 20200915 start
    public void SetScanKeyEnable(Context context, int keyCode, boolean enable) {
        Log.d(TAG, "SetScanKeyEnable" + keyCode + enable);
        SharedPreferences sharedPreferences = BaseSetting.getSharedPreferences(context);
        Editor editor = sharedPreferences.edit();
        String key = "";
        if (keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN) {
            key = ScanUtil.LEFT_SCAN_KEY;
        } else if (keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {
            key = ScanUtil.RIGHT_SCAN_KEY;
        } else if (keyCode == KeyEvent.KEYCODE_F11) {
            key = ScanUtil.FRONT_SCAN_KEY;
        }
        editor.putBoolean(key, enable);
        editor.commit();
    }

    public boolean isScanKeyEnable(Context context, int scankey) {
        boolean leftKeyEnable  =  BaseSetting.getSharedPreferences(context).getBoolean(ScanUtil.LEFT_SCAN_KEY,true);
        boolean rightKeyEnable  =  BaseSetting.getSharedPreferences(context).getBoolean(ScanUtil.RIGHT_SCAN_KEY,true);
        boolean frontKeyEnable  =  BaseSetting.getSharedPreferences(context).getBoolean(ScanUtil.FRONT_SCAN_KEY,true);

        if (scankey == KeyEvent.KEYCODE_XC_LEFT_SCAN && leftKeyEnable) {
            return true;
        }
        if (scankey == KeyEvent.KEYCODE_XC_RIGHT_SCAN && rightKeyEnable) {
            return true;
        }
        if (scankey == KeyEvent.KEYCODE_F11 && frontKeyEnable) {
            return true;
        }
        return false;
        //add by yangzhichao for t0581 20200915 end
    }
}
