package com.xcheng.scannere3;

import android.app.ActivityManager;
import android.app.ActivityManager.RunningTaskInfo;
import android.content.ComponentName;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.Intent;
import android.os.Environment;
//import android.os.ServiceManager;
import android.os.SystemProperties;
import android.text.TextUtils;
import android.util.Xml;
import android.util.Log;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.List;

import org.xmlpull.v1.XmlPullParser;
//add by yangzhichao for t0581 20200922 start
import com.xcheng.scannere3.codesetting.BaseSetting;
import com.xcheng.scannere3.XCScanner;
//add by yangzhichao for t0581 20200922 end
/**
 * This class define Scanner decode type
 */
public class ScanUtil {
    private final static String TAG = "ScanUtil";

    public static final int MAX_TYPE = 25;

    //modify by yangzhichao for t0581 20201021 start
    public static final int[] SP_TITLE_RES = {
        R.string.text_aztec_code, 
        R.string.text_code11, 
        R.string.text_code39, 
        R.string.text_code93,
        R.string.text_code128, 
        R.string.text_codabar, 
        R.string.text_codebolck_f, 
        R.string.text_data_matrix,
        R.string.text_ean_8, 
        R.string.text_ean_13, 
        R.string.text_gs1_databar, 
        R.string.text_han_xin,
        R.string.text_hk_25, 
        R.string.text_interleaved_25, 
        R.string.text_matrix_25, 
        R.string.text_maxicode,
        R.string.text_msi, 
        R.string.text_micro_pdf, 
        R.string.text_pdf417, /*R.string.text_postal,*/
        R.string.text_qr_code,
        R.string.text_industrial_25, 
        R.string.text_telepen, /*R.string.text_trioptic,*/
        R.string.text_upc_a, 
        R.string.text_upc_e, 
        R.string.text_gs1_128
    };
    //modify by yangzhichao for t0581 20201021 end

    public static final boolean[] SP_DEFAULT = {
        true, // aztec
        true, // code11
        true, // code39
        true, // code93
        true, // code128
        true, // codabar
        true, // codebolck_f
        true, // data_matrix
        true, // ean_8
        true, // ean_13
        true, // gs1_databar
        false, // han_xin
        true, // hk_25
        true, // interleaved_25
        true, // matrix_25
        true, // maxicode
        true, // msi
        true, // micro_pdf
        true, // pdf417
        true, // qr_code
        true, // industrial_25
        true, // telepen
        true, // upc_a
        true, // upc_e
        true  // gs1_128
    };

    private static final String KEY_SP_1 = "bc_aztec";
    private static final String KEY_SP_2 = "bc_code11";
    private static final String KEY_SP_3 = "bc_code39";
    private static final String KEY_SP_4 = "bc_code93";
    private static final String KEY_SP_5 = "bc_code128";
    private static final String KEY_SP_6 = "bc_codabar";
    private static final String KEY_SP_7 = "bc_codebolck_f";
    private static final String KEY_SP_8 = "bc_data_matrix";
    private static final String KEY_SP_9 = "bc_ean_8";
    private static final String KEY_SP_10 = "bc_ean_13";
    private static final String KEY_SP_11 = "bc_gs1_databar";
    private static final String KEY_SP_12 = "bc_han_xin";
    private static final String KEY_SP_13 = "bc_hk_25";
    private static final String KEY_SP_14 = "bc_interleaved_25";
    private static final String KEY_SP_15 = "bc_matrix_25";
    private static final String KEY_SP_16 = "bc_maxicode";
    private static final String KEY_SP_17 = "bc_msi";
    private static final String KEY_SP_18 = "bc_micro_pdf";
    private static final String KEY_SP_19 = "bc_pdf417";
    private static final String KEY_SP_20 = "bc_qr_code";
    private static final String KEY_SP_21 = "bc_industrial_25";
    private static final String KEY_SP_22 = "bc_telepen";
    private static final String KEY_SP_23 = "bc_upc_a";
    private static final String KEY_SP_24 = "bc_upc_e";
    private static final String KEY_SP_25 = "bc_gs1_128";

    public static final String[] SP_KEYS = {
        KEY_SP_1, 
        KEY_SP_2, 
        KEY_SP_3, 
        KEY_SP_4, 
        KEY_SP_5, 
        KEY_SP_6, 
        KEY_SP_7,
        KEY_SP_8, 
        KEY_SP_9, 
        KEY_SP_10, 
        KEY_SP_11, 
        KEY_SP_12, 
        KEY_SP_13, 
        KEY_SP_14,
        KEY_SP_15, 
        KEY_SP_16, 
        KEY_SP_17, 
        KEY_SP_18, 
        KEY_SP_19, 
        KEY_SP_20, 
        KEY_SP_21,
        KEY_SP_22, 
        KEY_SP_23, 
        KEY_SP_24, 
        KEY_SP_25
    };

    public static final int MAX_1D_TYPE = 17;
    public static final int MAX_2D_TYPE = 8;
    public static final int[] Scan_1D_Array = {
        //R.string.text_aztec_code, 
        R.string.text_code11, 
        R.string.text_code39, 
        R.string.text_code93,
        R.string.text_code128, 
        R.string.text_codabar, 
        //R.string.text_codebolck_f, 
        //R.string.text_data_matrix,
        R.string.text_ean_8, 
        R.string.text_ean_13, 
        R.string.text_gs1_databar, 
        //R.string.text_han_xin,
        R.string.text_hk_25, 
        R.string.text_interleaved_25, 
        R.string.text_matrix_25, 
        //R.string.text_maxicode,
        R.string.text_msi, 
        //R.string.text_micro_pdf, 
        //R.string.text_pdf417,
        //R.string.text_qr_code,
        R.string.text_industrial_25, 
        R.string.text_telepen,
        R.string.text_upc_a, 
        R.string.text_upc_e, 
        R.string.text_gs1_128
    };

    public static final int[] Scan_2D_Array = {
        R.string.text_aztec_code, 
        //R.string.text_code11, 
        //R.string.text_code39, 
        //R.string.text_code93,
        //R.string.text_code128, 
        //R.string.text_codabar, 
        R.string.text_codebolck_f, 
        R.string.text_data_matrix,
        //R.string.text_ean_8, 
        //R.string.text_ean_13, 
        //R.string.text_gs1_databar, 
        R.string.text_han_xin,
        //R.string.text_hk_25, 
        //R.string.text_interleaved_25, 
        //R.string.text_matrix_25, 
        R.string.text_maxicode,
        //R.string.text_msi, 
        R.string.text_micro_pdf, 
        R.string.text_pdf417,
        R.string.text_qr_code,
        //R.string.text_industrial_25, 
        //R.string.text_telepen,
        //R.string.text_upc_a, 
        //R.string.text_upc_e, 
        //R.string.text_gs1_128
    };

    private static final String KEY_QR_REVERSE = "qr_reverse";
    private static final String KEY_MQR_REVERSE = "qr_mqr_reverse";
    private static final String KEY_DM_REVERSE = "datamatrix_reverse";
    //add by yangzhichao for t0581 20200922 end

    //add by yangzhichao for t0581 20201009 start
    private static final String KEY_128_REVERSE = "code_128";
    //add by yangzhichao for t0581 20201009 end

    public static final String SCAN_DECODING_BROADCAST = "com.xcheng.scanner.action.BARCODE_DECODING_BROADCAST";
    public static final String SCAN_DECODING_DATA = "EXTRA_BARCODE_DECODING_DATA";
    public static final String SCAN_SYMBOLOGY_TYPE = "EXTRA_BARCODE_DECODING_SYMBOLE";

    public static final String DATA_RECEIVE_KEY = "pref_data_receive_method";

    //add by yangzhichao for t0581 20200915 start
    public static final String LEFT_SCAN_KEY = "pref_left_scankey";
    public static final String RIGHT_SCAN_KEY = "pref_right_scankey";
    public static final String FRONT_SCAN_KEY = "pref_front_scankey";
    public static final String SCANKEY = "scankey";

    public static final String ACTION_CONTROL_SCANKEY = "com.xcheng.scanner.action.ACTION_CONTROL_SCANKEY";
    public static final String EXTRA_SCANKEY_CODE = "extra_scankey_code";
    public static final String EXTRA_SCANKEY_STATUS = "extra_scankey_STATUS";
    //add by yangzhichao for t0581 20200915 end

    public static final String ACTION_OPEN_SCAN_BROADCAST = "com.xcheng.scanner.action.OPEN_SCAN_BROADCAST";
    public static final String ACTION_CLOSE_SCAN_BROADCAST = "com.xcheng.scanner.action.CLOSE_SCAN_BROADCAST";

    public static final String ACTION_ENABLE_TYPE = "com.xcheng.scanner.action.ENABLE_SCANTYPE_BROADCAST";
    public static final String ACTION_DISABLE_TYPE = "com.xcheng.scanner.action.DISABLE_SCANTYPE_BROADCAST";
    public static final String SCANTYPE = "scantype";
    //add by yangzhichao for t0581 20201021 start
    public static final String SCANTYPE_ALL_ON = "All On";
    public static final String SCANTYPE_ALL_OFF = "All Off";
    public static final String SCANTYPE_1D = "1D";
    public static final String SCANTYPE_2D = "2D";
    public static final String KEY_ALL_SWITCH = "swtich_all";
    public static final String KEY_1D_SWITCH = "swtich_1d";
    public static final String KEY_2D_SWITCH = "swtich_2d";
    //add by yangzhichao for t0581 20201021 end
    public static final String ACTION_CONTROL_DATA_EVENT = "com.xcheng.scanner.action.CONTROL_DATA_EVENT";
    public static final String EXTRA_DATA_EVENT = "extra_data_event";

    public static final String ACTION_LOAD_SETTINGS = "com.xcheng.scanner.action.LOAD_SETTINGS";
    public static final String ACTION_SAVE_SETTINGS = "com.xcheng.scanner.action.SAVE_SETTINGS";

    public static final String ACTION_START_SERVICE = "com.xcheng.scanner.action.START_SERVICE";
    public static final String ACTION_STOP_SERVICE = "com.xcheng.scanner.action.STOP_SERVICE";

    public static final String ACTION_SCANNER_ENGINE = "com.xcheng.scanner.action.SCANNER_ENGINE";
    public static final String EXTRA_SCANNER_ENGINE = "scanner_engine";

    public static final String ACTION_SCAN_TEST_ACTIVITY_ENTER = "com.xcheng.scanner.action.SCAN_TEST_ENTER";
    public static final String ACTION_SCAN_TEST_ACTIVITY_LEAVE = "com.xcheng.scanner.action.SCAN_TEST_LEAVE";

    public static final String ACTION_SCAN_TEST = "com.xcheng.scanner.action.SCAN_TEST";
    public static final String FACTORY_SCAN_TEST = "scan_test";

    public final static String XML_FILE_NAME = "com.xcheng.scannere3_preferences.xml";

    //add by yangzhichao for t0581 20200922 start
    public static void setScanType(String scanType, Context context) {
        if (scanType.equals(SCANTYPE_ALL_ON)) {
            //modify by yangzhichao for t0581 20201023 start
            //applyAllModulesSetting(context);
            setScan1DTypeStatus(true, context);
            setScan2DTypeStatus(true, context);
            //modify by yangzhichao for t0581 20201023 end
        } else if (scanType.equals(SCANTYPE_1D)) {
            setScan1DTypeStatus(true, context);
            setScan2DTypeStatus(false, context);
        } else if (scanType.equals(SCANTYPE_2D)) {
            setScan1DTypeStatus(false, context);
            setScan2DTypeStatus(true, context);
        } else if (scanType.equals(SCANTYPE_ALL_OFF)) {
            setScan1DTypeStatus(false, context);
            setScan2DTypeStatus(false, context);
        }
    }

    public static void setScan1DTypeStatus(boolean enable, Context context) {

        SharedPreferences mSharePref = BaseSetting.getSharedPreferences(context);
        //modify by yangzhichao for t0581 20200924 start
        for (int i = 0; i < MAX_1D_TYPE; i++) {
            E3Util.setCodeTypeOnOff(context.getString(Scan_1D_Array[i]), enable);
            mSharePref.edit().putBoolean(context.getString(Scan_1D_Array[i]), enable).commit();
        }
    }

    public static void setScan2DTypeStatus(boolean enable, Context context) {

        SharedPreferences mSharePref = BaseSetting.getSharedPreferences(context);
        for (int i = 0; i < MAX_2D_TYPE; i++) {
            E3Util.setCodeTypeOnOff(context.getString(Scan_2D_Array[i]), enable);
            mSharePref.edit().putBoolean(context.getString(Scan_2D_Array[i]), enable).commit();
        }
    }
        //modify by yangzhichao for t0581 20200924 end
    public static void setAllInverseCodeEnable(Context context, boolean enable){
        Editor editor = BaseSetting.getSharedPreferences(context).edit();
        editor.putInt(KEY_QR_REVERSE, enable ? 1 : 0);
        editor.putInt(KEY_MQR_REVERSE, enable ? 1 : 0);
        editor.putInt(KEY_DM_REVERSE, enable ? 1 : 0);
        //add by yangzhichao for t0581 20201009 start
        editor.putInt(KEY_128_REVERSE, enable ? 1 : 0);
        //add by yangzhichao for t0581 20201009 end
        editor.commit();

        int qrCfgValue = E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED);
        if (enable) {
            qrCfgValue |= (0x01<<1);
        } else {
            qrCfgValue &= (~(0x01<<1));
        }
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, qrCfgValue);

        int mqrCfgValue = E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED);
        if (enable) {
            mqrCfgValue |= (0x01<<3);
        } else {
            mqrCfgValue &= (~(0x01<<3));
        }
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, mqrCfgValue);

        int dmCfgValue = E3Util.getDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED);
        if (enable) {
            dmCfgValue |= (0x01<<1);
        } else {
            dmCfgValue &= (~(0x01<<1));
        }
        E3Util.setDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED, dmCfgValue);

        //add by yangzhichao for t0581 20201009 start
        //modify by yangzhichao for t0581 20201026 start
        /* Turn on affects performance ,So don't open it.
        int inverse1DBarCodeCfgValue = 0;
        if (enable) {
            inverse1DBarCodeCfgValue = 2;
        }
        E3Util.setDecoderTag(XCScanner.TAG_VIDEO_REVERSE_ENABLED, inverse1DBarCodeCfgValue);
        */
        //modify by yangzhichao for t0581 20201026 end
        int inverseC128CfgValue = 0;
        if (enable) {
            inverseC128CfgValue = 1;
        }
        E3Util.setDecoderTag(XCScanner.TAG_CODE128_INVERSE, inverseC128CfgValue);
        //add by yangzhichao for t0581 20201009 end
    }

    //add by yangzhichao for t0581 20200922 end
    public static String readFileData(String fileName) {
        String result = "";
        try {
            File file = new File(fileName);
            FileInputStream fis = new FileInputStream(file);
            int length = fis.available();
            byte[] buffer = new byte[length];
            fis.read(buffer);
            result = new String(buffer, "UTF-8");
        } catch (Exception e) {
            Log.d(TAG, "readFileData->e: " + e);
            e.printStackTrace();
        }
        return result;
    }

    public static void writeFileDataToSd(String toFileName, String fromFileName) {
        try {
            File file = new File(toFileName);
            FileOutputStream fos = new FileOutputStream(file);
            fos.write(readFileData(fromFileName).getBytes());
            fos.flush();
            fos.close();
        } catch (Exception e) {
            Log.d(TAG, "writeFileData->e: " + e);
            e.printStackTrace();
        }
    }

    public static String getSdCardPath() {
        boolean exist = isSdCardExist();
        Log.d(TAG, "getSdCardPath: " + exist);
        String sdPath = "/storage/emulated/0/";
        if (exist) {
            sdPath = Environment.getExternalStorageDirectory()
                    .getAbsolutePath();
        } else {
            sdPath = "/storage/emulated/0/";
        }
        return sdPath + "/";
    }

    private static boolean isSdCardExist() {
        return Environment.getExternalStorageState().equals(
                Environment.MEDIA_MOUNTED);
    }

    //add by yangzhichao for t0581 20201028 start
    public static void createSettingsXml(Context context) {
        String packageName = context.getPackageName();
        String filePath = "/data/data/" + packageName + "/shared_prefs/" + XML_FILE_NAME;
        try{
            if (!isFileExist(filePath)) {
                File file = new File(filePath);
                file.createNewFile();
                Log.d(TAG, "createSettingsXml->success");
            }
        } catch (Exception e) {
            Log.d(TAG, "createSettingsXml->e: " + e);
            e.printStackTrace();
        }

    }
    //add by yangzhichao for t0581 20201028 end
    public static void saveSettings(Context context) {
        String packageName = context.getPackageName();
        writeFileDataToSd(getSdCardPath() + XML_FILE_NAME,
                "/data/data/" + packageName + "/shared_prefs/" + XML_FILE_NAME);
    }

    public static void initSettings(Context context) {
        String packageName = context.getPackageName();
        loadSettings(context,
                "/data/data/" + packageName + "/shared_prefs/" + XML_FILE_NAME);
    }

    public static void loadSettings(Context context) {
        loadSettings(context, getSdCardPath() + XML_FILE_NAME);
    }

    private static void loadSettings(Context context, String fileName) {
        if (!isFileExist(fileName)) {
            Log.d(TAG, "scannere3_preferences is not exist");
            return;
        }
        SharedPreferences sharedPreferences = getSharedPreferences(context);
        Editor editor = sharedPreferences.edit();

        try {
            XmlPullParser pullParser = Xml.newPullParser();
            InputStream is = new ByteArrayInputStream(readFileData(fileName).getBytes("UTF-8"));
            pullParser.setInput(is, "utf-8");
            int eventType = pullParser.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT) {
                String tagName = pullParser.getName();

                // Parse "string" items
                if (!TextUtils.isEmpty(tagName) && "string".equals(tagName)) {
                    if (eventType == XmlPullParser.START_TAG) {
                        String nameString = pullParser.getAttributeValue(null, "name");
                        String valueString = pullParser.nextText();
                        if (!TextUtils.isEmpty(valueString)) {
                            editor.putString(nameString, valueString);
                            editor.commit();
                        }
                    }
                }

                // Parse "boolean" items
                if (!TextUtils.isEmpty(tagName) && "boolean".equals(tagName)) {
                    if (eventType == XmlPullParser.START_TAG) {
                        String nameBoolean = pullParser.getAttributeValue(null, "name");
                        String valueBoolean = pullParser.getAttributeValue(null, "value");
                        if (!TextUtils.isEmpty(valueBoolean)) {
                            boolean enabled = Boolean.parseBoolean(valueBoolean);
                            editor.putBoolean(nameBoolean, enabled);
                            editor.commit();
                        }
                    }
                }

                // Parse "int" items
                if (!TextUtils.isEmpty(tagName) && "int".equals(tagName)) {
                    if (eventType == XmlPullParser.START_TAG) {
                        String nameInt = pullParser.getAttributeValue(null, "name");
                        String valueInt = pullParser.getAttributeValue(null, "value");
                        if (!TextUtils.isEmpty(valueInt)) {
                            editor.putInt(nameInt, Integer.parseInt(valueInt));
                            editor.commit();
                        }
                    }
                }
                eventType = pullParser.next();
            }

            // Configurations need sync to scan decode engine
            //add by yangzhichao for t0581 20200924 start
            boolean stateAll = sharedPreferences.getBoolean(KEY_ALL_SWITCH, true);
            boolean state1D = sharedPreferences.getBoolean(KEY_1D_SWITCH, false);
            boolean state2D = sharedPreferences.getBoolean(KEY_2D_SWITCH, false);
            if(stateAll){
                setScanType(SCANTYPE_ALL_ON ,context);
            }else if (state1D && state2D){
                setScanType(SCANTYPE_ALL_ON ,context);
            }else if(state1D){
                setScanType(KEY_1D_SWITCH ,context);
            }else if(state2D){
                setScanType(KEY_2D_SWITCH ,context);
            }else{
                for (int i = 0;i < MAX_TYPE; i++){
                    E3Util.setCodeTypeOnOff(context.getString(SP_TITLE_RES[i]),
                        sharedPreferences.getBoolean(SP_KEYS[i], false));
                }
            }

            //Inverse Code Enable
            //modify by yangzhichao for t0581 20201026 start
            /* Turn on affects performance ,So don't open it.
            boolean inverseCode = sharedPreferences.getBoolean("pref_inverse_code_enable", false);

            E3Util.setDecoderTag(XCScanner.TAG_VIDEO_REVERSE_ENABLED,
                inverseCode ?
                E3Util.getDecoderTag(XCScanner.TAG_VIDEO_REVERSE_ENABLED) | (0x01<<1) :
                E3Util.getDecoderTag(XCScanner.TAG_VIDEO_REVERSE_ENABLED) & (~(0x01<<1)));
            Log.d(TAG, "Inverse Code Enable" + inverseCode);
            */
            //modify by yangzhichao for t0581 20201026 end
            //add by yangzhichao for t0581 20200924 end
            // Timeout value
            String maxTimeoutString = sharedPreferences.getString("pref_max_scan_time", "2000");
            if (isValidTimeout(context, maxTimeoutString)) {
                E3Util.setTimeout(Integer.parseInt(maxTimeoutString));
            }

            // Scan mode
            String workMode = sharedPreferences.getString("pref_scan_workmode", "0");
            E3Util.setScanMode(Integer.parseInt(workMode));

            // Scan viewsize
            String viewSize = sharedPreferences.getString("pref_scan_viewsize", "0");
            E3Util.configViewSize(Integer.parseInt(viewSize));

            // FLASH light
            boolean lightFlashEnable = sharedPreferences.getBoolean("pref_light_flash_enable", true);
            E3Util.setLightFlashSupport(lightFlashEnable);

            // AIM light
            boolean lightAimEnable = sharedPreferences.getBoolean("pref_light_aim_enable", true);
            E3Util.setLightAimSupport(lightAimEnable);

            //modify by yangzhichao for t0581 20200924 start
            // Code types
            //applyAllModulesSetting(context);
            //modify by yangzhichao for t0581 20200924 end

            //add by yangzhichao for t0581 20201014 start
            applyAllSettingsDetailSetting(context);
            //add by yangzhichao for t0581 20201014 end
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    // Apply all code format support settings
    // If switch_all true, not need to apply detailed settings
    // If switch_all false, need to enable code format of AllModulesSetting enabled items.
    public static void applyAllModulesSetting(Context context) {
        SharedPreferences preferences = getSharedPreferences(context);
        
        // Config switch_all value
        boolean swtich_all_on = preferences.getBoolean("swtich_all", true);
        if (swtich_all_on) {
            E3Util.enableAllCodeTypes();
        } else {
            E3Util.disableAllCodeTypes();
        }

        // Config all configs if switch_all is off
        if (!swtich_all_on) {
            int i = 0;
            for (i = 0; i < SP_KEYS.length; i++) {
                boolean symonoff = preferences.getBoolean(SP_KEYS[i], false);

                if (symonoff) {
                    E3Util.setCodeTypeOnOff(context.getString(SP_TITLE_RES[i]), symonoff);
                }
            }
        }
    }

    public static void clear(Context context) {
        restoreDefaultSymbology(context);
        SharedPreferences preferences = getSharedPreferences(context);
        Editor editor = preferences.edit();
        editor.clear();
        editor.commit();
    }

    public static void restoreDefaultSymbology(Context context) {
        for (int i = 0; i < MAX_TYPE; i++) {
            E3Util.setCodeTypeOnOff(context.getString(SP_TITLE_RES[i]), SP_DEFAULT[i]);
        }
    }

    public static boolean isValidTimeout(Context context, String timeout) {
        String[] timeoutValues = context.getResources().getStringArray(R.array.scan_timeout_values);
        for (int i = 0; i < timeoutValues.length; i++) {
            if (timeoutValues[i].equals(timeout)) {
                return true;
            }
        }
        return false;
    }

    public static String getSymbologyByKey(Context context, String key) {
        String symbology = "";
        for (int i = 0; i < MAX_TYPE; i++) {
            if (SP_KEYS[i].equals(key)) {
                symbology = context.getString(SP_TITLE_RES[i]);
                break;
            }
        }
        return symbology;
    }

    public static SharedPreferences getSharedPreferences(Context context){
        return context.getSharedPreferences("com.xcheng.scannere3_preferences", Context.MODE_PRIVATE);
    }

    public static boolean needModifyGsCharacter(Context context, String result) {
        boolean enabled = getSharedPreferences(context)
                .getBoolean(ScanSettingActivity.ENABLE_CHARACTER_MODIFY,
                        ScanSettingActivity.DEFAULT_GS_MODIFY_ENABLE);
        Log.d(TAG, "Modify GS enabled: " + enabled);
        if (enabled) {
            return result.indexOf((char)0x1d) != -1;
        }
        return enabled;
    }

    public static String getStringForGs(Context context) {
        String gs = getSharedPreferences(context)
                .getString(ScanSettingActivity.CHARACTER_GS, ScanSettingActivity.EMPTY);
        Log.d(TAG, "gs: " + gs);
        return gs;
    }

    public static String replaceGsCharacter(Context context, String result) {
        String gs = getStringForGs(context);
        if (!(ScanSettingActivity.EMPTY.equals(gs))) {
            if (gs.equals("TAB")) {
                result = result.replace((char)0x1d, (char)0x09);
            } else if (gs.equals("ENTER")) {
                result = result.replace((char)0x1d, (char)0x0d);
            } else if (gs.equals("SPACE")) {
                result = result.replace((char)0x1d, (char)0x20);
            } else {
                result = result.replace((char)0x1d, gs.charAt(0));
            }
            Log.d(TAG, "GS Data: " + result);
        }
        return result;
    }

    public static boolean isFileExist(String fileName) {
        boolean isExist = false;
        try {
            File file = new File(fileName);
            if (file.exists()) {
                isExist = true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        Log.d(TAG, "isExist = " + isExist);
        return isExist  ;
    }

    // Check if this class of context allow scan function
    public static boolean isActivityAllowScan(Context context) {
        ActivityManager am = (ActivityManager)context.getSystemService(Context.ACTIVITY_SERVICE);
        List<RunningTaskInfo> runningTasks = am.getRunningTasks(1);
        if (runningTasks != null && runningTasks.size() > 0) {
            ComponentName comName = runningTasks.get(0).topActivity;

//            Log.d(TAG, "PackageName = " + comName.getPackageName());
//            Log.d(TAG, "ClassName = " + comName.getClassName());

            // Any other package allow scan
            if (!"com.xcheng.scannere3".equals(comName.getPackageName())) {
                return true;
            } else if ("com.xcheng.scannere3.scantest.ScanActivity".equals(comName.getClassName())) {
                // If in ScannerE3 package, and in ScanActivity allow scan
                return true;
            } else if ("com.xcheng.scannere3.scantest.ScanContinueActivity".equals(comName.getClassName())) {
                // If in ScannerE3 package, and in ScanContinueActivity allow scan
                return true;
            }
        }
        
        return false;
    }

    public static void openCloseDevice(Context context, boolean isOpen){
        Intent intent = new Intent();
        if (isOpen) {
            intent.setAction(ACTION_OPEN_SCAN_BROADCAST);
        } else {
            intent.setAction(ACTION_CLOSE_SCAN_BROADCAST);
        }
        context.sendBroadcast(intent);
        //context.sendOrderedBroadcast(intent, null);
    }
    //add by yangzhichao for t0581 20201014 start
    private static void applyAllSettingsDetailSetting(Context context){
        Log.e(TAG,"applyAllSettingsDetailSetting");
        //AZTEC
        E3Util.setDecoderTag(XCScanner.TAG_AZTEC_ENABLED, getSharedPreferences(context).getInt("aztec_inverce", 1) == 1 ? 3 : 1);
        E3Util.setDecoderTag(XCScanner.TAG_AZTEC_MAX_LENGTH, getSharedPreferences(context).getInt("aztec_max", 3832));
        E3Util.setDecoderTag(XCScanner.TAG_AZTEC_MIN_LENGTH, getSharedPreferences(context).getInt("aztec_min", 1));
        //C11
        E3Util.setDecoderTag(XCScanner.TAG_CODE11_CHECK_DIGIT_MODE, getSharedPreferences(context).getInt("code11_check_digit", 4));
        E3Util.setDecoderTag(XCScanner.TAG_CODE11_MAX_LENGTH, getSharedPreferences(context).getInt("code11_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_CODE11_MIN_LENGTH, getSharedPreferences(context).getInt("code11_min", 6));
        //C39
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_START_STOP_TRANSMIT, getSharedPreferences(context).getInt("code39_transmit", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_APPEND_ENABLED, getSharedPreferences(context).getInt("code39_append", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_CHECK_DIGIT_MODE, getSharedPreferences(context).getInt("code39_verify", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_FULL_ASCII_ENABLED, getSharedPreferences(context).getInt("code39_full_asc", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_BASE32_ENABLED, getSharedPreferences(context).getInt("code39_convert_to_code32", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_MIN_LENGTH, getSharedPreferences(context).getInt("code39_min", 4));
        E3Util.setDecoderTag(XCScanner.TAG_CODE39_MAX_LENGTH, getSharedPreferences(context).getInt("code39_max", 127));
        //C93
        E3Util.setDecoderTag(XCScanner.TAG_CODE93_MAX_LENGTH, getSharedPreferences(context).getInt("code93_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_CODE93_MIN_LENGTH, getSharedPreferences(context).getInt("code93_min", 2));
        //C128
        E3Util.setDecoderTag(XCScanner.TAG_CODE128_INVERSE, getSharedPreferences(context).getInt("code_128", 1) == 1 ? (E3Util.getDecoderTag(XCScanner.TAG_CODE128_INVERSE) | (0x01)) : (E3Util.getDecoderTag(XCScanner.TAG_CODE128_INVERSE) & (~(0x01))));
        E3Util.setDecoderTag(XCScanner.TAG_C128_ISBT_ENABLED, getSharedPreferences(context).getInt("code128_isbt_128", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_CODE128_MAX_LENGTH, getSharedPreferences(context).getInt("code128_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_CODE128_MIN_LENGTH, getSharedPreferences(context).getInt("code128_min", 1));
        //CODABAR
        E3Util.setDecoderTag(XCScanner.TAG_CODABAR_CHECK_DIGIT_MODE, getSharedPreferences(context).getInt("transmit_check_digit", 0) == 0 ? 0 : 2);
        E3Util.setDecoderTag(XCScanner.TAG_CODABAR_MAX_LENGTH, getSharedPreferences(context).getInt("codebar_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_CODABAR_MIN_LENGTH, getSharedPreferences(context).getInt("codebar_min", 4));
        //CODEBLACK F
        E3Util.setDecoderTag(XCScanner.TAG_CODABLOCK_F_MAX_LENGTH, getSharedPreferences(context).getInt("codeblock_f_max", 55));
        E3Util.setDecoderTag(XCScanner.TAG_CODABLOCK_F_MIN_LENGTH, getSharedPreferences(context).getInt("codeblock_f_min", 0));

        //DATA MATRIX
        int tag_value = E3Util.getDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED);
        if (getSharedPreferences(context).getInt("datamatrix_reverse", 1) == 1) {
            tag_value |= (0x01<<1);
        } else {
            tag_value &= (~(0x01<<1));
        }
        if (getSharedPreferences(context).getInt("datamatrix_mirror", 1) == 1) {
            tag_value |= (0x01<<6);
        } else {
            tag_value &= (~(0x01<<6));
        }
        E3Util.setDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED, tag_value);
        E3Util.setDecoderTag(XCScanner.TAG_DATAMATRIX_MAX_LENGTH, getSharedPreferences(context).getInt("datamatrix_min", 3116));
        E3Util.setDecoderTag(XCScanner.TAG_DATAMATRIX_MIN_LENGTH, getSharedPreferences(context).getInt("datamatrix_max", 1));

        //GS1 128
        E3Util.setDecoderTag(XCScanner.TAG_GS1_128_MAX_LENGTH, getSharedPreferences(context).getInt("gs1_128_max", 80));
        E3Util.setDecoderTag(XCScanner.TAG_GS1_128_MIN_LENGTH, getSharedPreferences(context).getInt("gs1_128_min", 0));
        //GS1 DATABAR(RSS)
        E3Util.setDecoderTag(XCScanner.TAG_RSS_EXPANDED_ENABLED, getSharedPreferences(context).getInt("gs1_expanded", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_RSS_LIMITED_ENABLED, getSharedPreferences(context).getInt("gs1_limited", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_RSS_EXPANDED_MAX_LENGTH, getSharedPreferences(context).getInt("gs1expanded_max", 128));
        E3Util.setDecoderTag(XCScanner.TAG_RSS_EXPANDED_MIN_LENGTH, getSharedPreferences(context).getInt("gs1expanded_min", 1));

        //HANXIN
        E3Util.setDecoderTag(XCScanner.TAG_HANXIN_MAX_LENGTH, getSharedPreferences(context).getInt("hanxin_max", 7827));
        E3Util.setDecoderTag(XCScanner.TAG_HANXIN_MIN_LENGTH, getSharedPreferences(context).getInt("hanxin_min", 1));
        //HK25
        E3Util.setDecoderTag(XCScanner.TAG_HK25_MAX_LENGTH, getSharedPreferences(context).getInt("hk25_max", 55));
        E3Util.setDecoderTag(XCScanner.TAG_HK25_MIN_LENGTH, getSharedPreferences(context).getInt("hk25_min", 5));
        //INTERLEAVED 25
        E3Util.setDecoderTag(XCScanner.TAG_I25_CHECK_DIGIT_MODE, getSharedPreferences(context).getInt("interleaved2of5_transmit", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_I25_MAX_LENGTH, getSharedPreferences(context).getInt("interleaved2of5_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_I25_MIN_LENGTH, getSharedPreferences(context).getInt("interleaved2of5_min", 6));
        //MATRIX 25
        E3Util.setDecoderTag(XCScanner.TAG_M25_CHECK_DIGIT_MODE, getSharedPreferences(context).getInt("matrix2of5_transmit", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_M25_MAX_LENGTH, getSharedPreferences(context).getInt("matrix2of5_max", 127));
        E3Util.setDecoderTag(XCScanner.TAG_M25_MIN_LENGTH, getSharedPreferences(context).getInt("matrix2of5_min", 6));
        //MATRIX CODE
        E3Util.setDecoderTag(XCScanner.TAG_MAXICODE_MAX_LENGTH, getSharedPreferences(context).getInt("maxi_code_max", 150));
        E3Util.setDecoderTag(XCScanner.TAG_MAXICODE_MIN_LENGTH, getSharedPreferences(context).getInt("maxi_code_min", 1));
        //MICRO PDF
        E3Util.setDecoderTag(XCScanner.TAG_MICROPDF_MAX_LENGTH, getSharedPreferences(context).getInt("micro_pdf_max", 250));
        E3Util.setDecoderTag(XCScanner.TAG_MICROPDF_MIN_LENGTH, getSharedPreferences(context).getInt("micro_pdf_min", 5));
        
        //QR CODE
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, getSharedPreferences(context).getInt("qr_reverse", 1) == 1 ? E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) | (0x01<<1) : E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) & (~(0x01<<1)));
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, getSharedPreferences(context).getInt("qr_mqr", 1) == 1 ? (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) | (0x01<<2)) : (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) & (~(0x01<<2))));
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, getSharedPreferences(context).getInt("qr_mqr_reverse", 1) == 1 ? (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) | (0x01<<3)) : (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) & (~(0x01<<3))));
        E3Util.setDecoderTag(XCScanner.TAG_QR_ENABLED, getSharedPreferences(context).getInt("qr_mirror", 1) == 1 ? (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) | (0x01<<4)) : (E3Util.getDecoderTag(XCScanner.TAG_QR_ENABLED) & (~(0x01<<4))));
        E3Util.setDecoderTag(XCScanner.TAG_QR_MAX_LENGTH, getSharedPreferences(context).getInt("qrcode_max", 7089));
        E3Util.setDecoderTag(XCScanner.TAG_QR_MIN_LENGTH, getSharedPreferences(context).getInt("qrcode_min", 1));
        //PDF417
        E3Util.setDecoderTag(XCScanner.TAG_MICROPDF_MAX_LENGTH, getSharedPreferences(context).getInt("pdf417_max", 366));
        E3Util.setDecoderTag(XCScanner.TAG_MICROPDF_MIN_LENGTH, getSharedPreferences(context).getInt("pdf417_min", 1));
        //EAN-8
        E3Util.setDecoderTag(XCScanner.TAG_EAN8_CHECK_DIGIT_TRANSMIT, getSharedPreferences(context).getInt("ean8_transmit_check", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN8_2CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("ean8_2char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN8_5CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("ean8_5char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN8_ADDENDA_REQUIRED, getSharedPreferences(context).getInt("ean8_required_addenda", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN8_ADDENDA_SEPARATOR, getSharedPreferences(context).getInt("ean8_separator_addenda", 1) == 1 ? 1 : 0);
        //EAN-13
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_CHECK_DIGIT_TRANSMIT, getSharedPreferences(context).getInt("ean13_transmit_check", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_2CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("ean13_2char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_5CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("ean13_5char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_ADDENDA_REQUIRED, getSharedPreferences(context).getInt("ean13_required_addenda", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_ADDENDA_SEPARATOR, getSharedPreferences(context).getInt("ean13_separator_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_EAN13_ISBN_ENABLED, getSharedPreferences(context).getInt("ean13_isbn_enable", 0) == 1 ? 1 : 0);
        //UPC-A
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_CHECK_DIGIT_TRANSMIT, getSharedPreferences(context).getInt("upca_transmit_check", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_NUMBER_SYSTEM_TRANSMIT, getSharedPreferences(context).getInt("upca_number_system", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_2CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("upca_2char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_5CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("upca_5char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_ADDENDA_REQUIRED, getSharedPreferences(context).getInt("upca_required_addenda", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_ADDENDA_SEPARATOR, getSharedPreferences(context).getInt("upca_separator_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCA_ADD_COUNTRY_CODE, getSharedPreferences(context).getInt("upca_country_code", 0) == 1 ? 1 : 0);
        //UPC-E
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_EXPAND, getSharedPreferences(context).getInt("upce_expand", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_CHECK_DIGIT_TRANSMIT, getSharedPreferences(context).getInt("upce_transmit_check", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_NUMBER_SYSTEM_TRANSMIT, getSharedPreferences(context).getInt("upce_number_system", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_2CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("upce_2char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_5CHAR_ADDENDA_ENABLED, getSharedPreferences(context).getInt("upce_5char_addenda", 1) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_ADDENDA_REQUIRED, getSharedPreferences(context).getInt("upce_required_addenda", 0) == 1 ? 1 : 0);
        E3Util.setDecoderTag(XCScanner.TAG_UPCE_ADDENDA_SEPARATOR, getSharedPreferences(context).getInt("upce_separator_addenda", 1) == 1 ? 1 : 0);
        //TELEPEN
        E3Util.setDecoderTag(XCScanner.TAG_TELEPEN_MAX_LENGTH, getSharedPreferences(context).getInt("telepen_max", 60));
        E3Util.setDecoderTag(XCScanner.TAG_TELEPEN_MIN_LENGTH, getSharedPreferences(context).getInt("telepen_min", 0));


    }
    //add by yangzhichao for t0581 20201014 end
}
