package com.xcheng.scannere3.codesetting;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.PreferenceActivity;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import android.text.InputType;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.view.KeyEvent;
import android.widget.EditText;
import android.widget.Toast;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.E3Util;
import com.xcheng.scannere3.ScanUtil;
//add by yangzhihcao for t0581 20201027 start
import android.text.InputFilter;
//add by yangzhihcao for t0581 20201027 end

public class BaseSetting extends PreferenceActivity {

    private static final String TAG = "BaseSetting";

    private static final String MAX_LENGTH_TEXT = "MAX_LENGTH";
    private static final String MIN_LENGTH_TEXT = "MIN_LENGTH";
    private static final int SEND_COMMAND = 0x01;

    public static final int TYPE_PREFERENCE = 0;
    public static final int TYPE_SWTICH = 1;
    public static final int TYPE_LIST = 2;

    private EditText mMaxEditText;
    private EditText mMinEditText;
    private String mMaxText;
    private String mMinText;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.base_setting);
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        if (mMaxEditText != null) {
            outState.putString(MAX_LENGTH_TEXT, mMaxEditText.getText().toString());
        }
        if (mMinEditText != null) {
            outState.putString(MIN_LENGTH_TEXT, mMinEditText.getText().toString());
        }
    }

    @Override
    public void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        if (savedInstanceState != null && savedInstanceState.containsKey(MAX_LENGTH_TEXT)) {
            mMaxText = savedInstanceState.getString(MAX_LENGTH_TEXT);
        }
        if (savedInstanceState != null && savedInstanceState.containsKey(MIN_LENGTH_TEXT)) {
            mMinText = savedInstanceState.getString(MIN_LENGTH_TEXT);
        }
    }

    private void setSummaryText(String key, String text) {
        if (TextUtils.isEmpty(text)) {
            text = getString(R.string.text_unknown);
        }
        // some preferences may be missing
        if (findPreference(key) != null) {
            findPreference(key).setSummary(text);
        }
    }

    private void removePreferenceFromScreen(String key) {
        Preference pref = findPreference(key);
        if (pref != null) {
            getPreferenceScreen().removePreference(pref);
        }
    }

    public void updateListPreferenceSummary(ListPreference preference, long current) {
        String summary;
        if (current < 0) {
            // Unsupported value
            summary = "";
        } else {
            final CharSequence[] entries = preference.getEntries();
            final CharSequence[] values = preference.getEntryValues();
            if (entries == null || entries.length == 0) {
                summary = "";
            } else {
                int best = 0;
                for (int i = 0; i < values.length; i++) {
                    long index = Long.parseLong(values[i].toString());
                    if (current >= index) {
                        best = i;
                    }
                }
                summary = entries[best].toString();
            }
        }
        preference.setSummary(summary);
    }

    public Preference createPreference(Context context, int type,
            String key, String title, int parm) {
        Preference preference = null;
        //int data = E3Util.getNumParameter(parm);
        int data = getDecoderTagValue(parm);
        Log.d(TAG, "parm = " + parm + ", data = " + data);
        switch (type) {
            case TYPE_PREFERENCE:
                preference = new Preference(context);
                int value = getSharedPreferences(this).getInt(key, -1);
                if (value == -1){
                    getSharedPreferences(context).edit().putInt(key, data).commit();
                }
                break;
            case TYPE_SWTICH:
                preference = new SwitchPreference(context);
                int checked = getSharedPreferences(this).getInt(key, -1);
                if (checked == -1){
                    getSharedPreferences(context).edit().putInt(key, data).commit();
                }
                break;
            case TYPE_LIST:
                preference = new ListPreference(context);
                int valueList = getSharedPreferences(this).getInt(key, -1);
                if (valueList == -1){
                    getSharedPreferences(context).edit().putInt(key, data).commit();
                }
                break;
            default:
                break;
        }

        preference.setTitle(title);
        getPreferenceScreen().addPreference(preference);
        return preference;
    }

    public void createMaxLengthDialog(Context context, final Preference preference,
            String title, final int max, final int min, final int parm) {
        mMaxEditText = new EditText(context);
        mMaxEditText.setInputType(InputType.TYPE_CLASS_NUMBER);
        mMaxEditText.setTextDirection(View.TEXT_DIRECTION_LOCALE);
        //modify by yangzhihcao for t0581 20201027 start
        InputFilter[] filters = {new InputFilter.LengthFilter(5)};
        mMaxEditText.setFilters(filters);
        //modify by yangzhihcao for t0581 20201027 end
        mMaxText = Integer.toString(getSharedPreferences(this).getInt(preference.getKey(), 0));
        if (mMaxText != null) {
            mMaxEditText.setText(mMaxText);
            mMaxEditText.setSelection(mMaxText.length());
        }
        final String oldText = mMaxText;
        mMaxText = null;
        AlertDialog alertDialog = new AlertDialog.Builder(context)
                .setTitle(title)
                .setNegativeButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // TODO Auto-generated method stub
                        if (!TextUtils.isEmpty(mMaxEditText.getText())) {
                            mMaxText = mMaxEditText.getText().toString();
                            //int length = Integer.parseInt(Integer.toString(new Integer(mMaxText)));
                            //if (length >= min && length <= max) {
                                preference.setSummary(Integer.toString(new Integer(mMaxText)));
                                Editor editor = getSharedPreferences(BaseSetting.this).edit();
                                editor.putInt(preference.getKey(), new Integer(mMaxText));
                                editor.commit();
                                //E3Util.setParameter(parm, new Integer(mMaxText));
                                int ret = setDecoderTag(parm, new Integer(mMaxText));
                                Log.d(TAG, "createMaxLengthDialog ret = " + ret);
                            /*} else {
                                mMaxText = oldText;
                                Toast.makeText(BaseSetting.this,
                                        R.string.text_out_of_range,
                                        Toast.LENGTH_SHORT).show();
                            }*/
                        }
                        dialog.dismiss();
                    }
                })
                .setPositiveButton(android.R.string.cancel, null)
                .create();
        alertDialog.setView(mMaxEditText);
        alertDialog.setCanceledOnTouchOutside(false);
        alertDialog.setOnShowListener(new DialogInterface.OnShowListener() {
            public void onShow(DialogInterface dialog) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.showSoftInput(mMaxEditText, InputMethodManager.SHOW_IMPLICIT);
            }
        });
        alertDialog.show();
    }

    public void createMinLengthDialog(Context context, final Preference preference,
            String title, final int max, final int min, final int parm) {
        mMinEditText = new EditText(context);
        mMinEditText.setInputType(InputType.TYPE_CLASS_NUMBER);
        mMinEditText.setTextDirection(View.TEXT_DIRECTION_LOCALE);
        //modify by yangzhihcao for t0581 20201027 start
        InputFilter[] filters = {new InputFilter.LengthFilter(5)};
        mMinEditText.setFilters(filters);
        //modify by yangzhihcao for t0581 20201027 end
        mMinText = Integer.toString(getSharedPreferences(this).getInt(preference.getKey(), 0));
        if (mMinText != null) {
            mMinEditText.setText(mMinText);
            mMinEditText.setSelection(mMinText.length());
        }
        final String oldText = mMinText;
        mMinText = null;
        AlertDialog alertDialog = new AlertDialog.Builder(context)
                .setTitle(title)
                .setNegativeButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        // TODO Auto-generated method stub
                        if (!TextUtils.isEmpty(mMinEditText.getText())) {
                            mMinText = mMinEditText.getText().toString();
                            //int length = Integer.parseInt(mMinText);
                            //if (length >= min && length <= max) {
                                preference.setSummary(Integer.toString(new Integer(mMinText)));
                                Editor editor = getSharedPreferences(BaseSetting.this).edit();
                                editor.putInt(preference.getKey(), new Integer(mMinText));
                                editor.commit();
                                //E3Util.setParameter(parm, new Integer(mMinText));
                                int ret = setDecoderTag(parm, new Integer(mMinText));
                                Log.d(TAG, "createMinLengthDialog ret = " + ret);
                            /*} else {
                                mMinText = oldText;
                                Toast.makeText(BaseSetting.this,
                                        R.string.text_out_of_range,
                                        Toast.LENGTH_SHORT).show();
                            }*/
                        }
                        dialog.dismiss();
                    }
                })
                .setPositiveButton(android.R.string.cancel, null)
                .create();
        alertDialog.setView(mMinEditText);
        alertDialog.setCanceledOnTouchOutside(false);
        alertDialog.setOnShowListener(new DialogInterface.OnShowListener() {
            public void onShow(DialogInterface dialog) {
                InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.showSoftInput(mMinEditText, InputMethodManager.SHOW_IMPLICIT);
            }
        });
        alertDialog.show();
    }

    public static SharedPreferences getSharedPreferences(Context context){
        return ScanUtil.getSharedPreferences(context);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_VOLUME_DOWN
                || keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_VOLUME_DOWN
                || keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
            return true;
        }
        return super.onKeyUp(keyCode, event);
    }

    public static int setParameter(int paramNum, int paramVal) {
        return E3Util.setParameter(paramNum, paramVal);
    }

    public static int setDecoderTag(int tag, int value) {
        return E3Util.setDecoderTag(tag, value);
    }

    public static int getDecoderTagValue(int tag) {
        return E3Util.getDecoderTag(tag);
    }
}
