package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
//add by yangzhichao for t0581 20201016 start
import com.xcheng.scannere3.XCScanner;
//add by yangzhichao for t0581 20201016 end

public class Code11Setting extends BaseSetting {

    private static final String TAG = "Code11Setting";

    private static final int CHECK_DIGIT_PARM = 52;
    private static final int TRANSMIT_CHECK_DIGIT_PARM = 47;
    private static final int MIN_LENGTH_PARM = 28;
    private static final int MAX_LENGTH_PARM = 29;

    private static final String KEY_CHECK_DIGIT = "code11_check_digit";
    private static final String KEY_TRANSMIT_CHECK_DIGIT = "code11_transmit";
    private static final String KEY_MIN = "code11_min";
    private static final String KEY_MAX = "code11_max";

    private static final int MAX_LENGTH = 127;
    private static final int MIN_LENGTH = 6;
    private static final int DFAULT_MIN_LENGTH = 4;

    private ListPreference mDigitChekPref;
    private SwitchPreference mTransmitPref;
    private Preference mMaxPref;
    private Preference mMinPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //modify by yangzhichao for t0581 20201016 start
        mDigitChekPref = (ListPreference) createPreference(this, TYPE_LIST,
                KEY_CHECK_DIGIT, getString(R.string.check_digit), CHECK_DIGIT_PARM);
        mDigitChekPref.setEntries(R.array.check_digit_setting_entries);
        mDigitChekPref.setEntryValues(R.array.check_digit_setting_values);
        mDigitChekPref.setDialogTitle(R.string.check_digit);
        //int value = BaseSetting.getSharedPreferences(this).getInt(KEY_CHECK_DIGIT, -1);
        int value = getDecoderTagValue(XCScanner.TAG_CODE11_CHECK_DIGIT_MODE);
        
        if (value != -1){
            mDigitChekPref.setValueIndex(value);
            updateListPreferenceSummary(mDigitChekPref, value);
        }
        mDigitChekPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                int valueSelect = new Integer(String.valueOf(objValue));
                Editor editor = BaseSetting.getSharedPreferences(Code11Setting.this).edit();
                editor.putInt(KEY_CHECK_DIGIT, valueSelect);
                editor.commit();
                setParameter(CHECK_DIGIT_PARM, valueSelect);
                updateListPreferenceSummary((ListPreference)preference,
                        Long.parseLong((String) objValue));
                setDecoderTag(XCScanner.TAG_CODE11_CHECK_DIGIT_MODE, valueSelect);
                return true;
            }
        });
        /*
        mTransmitPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT_CHECK_DIGIT, getString(R.string.transmit_check_digit), TRANSMIT_CHECK_DIGIT_PARM);
        mTransmitPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT_CHECK_DIGIT, -1) == 1);
        mTransmitPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code11Setting.this).edit();
                editor.putInt(KEY_TRANSMIT_CHECK_DIGIT, checked ? 1 : 0);
                editor.commit();
                setParameter(TRANSMIT_CHECK_DIGIT_PARM, checked ? 1 : 0);
                return true;
            }
        });
        */
        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_CODE11_MAX_LENGTH);
        //int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue = getDecoderTagValue(XCScanner.TAG_CODE11_MAX_LENGTH);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Code11Setting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE11_MAX_LENGTH);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_CODE11_MIN_LENGTH);
        //int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue = getDecoderTagValue(XCScanner.TAG_CODE11_MIN_LENGTH);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(DFAULT_MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Code11Setting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE11_MIN_LENGTH);
                return true;
            }
        });
        
    //modify by yangzhichao for t0581 20201016 end
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
