package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;


public class Code128Setting extends BaseSetting {

    private static final String TAG = "Code128Setting";

    private static final int GS1_128_PARM = 14;
    private static final int ISBT_128_PARM = 84;
    private static final int MAX_PARM = 210;
    private static final int MIN_PARM = 209;

    private static final String KEY_GS1_128 = "code128_gs1_128";
    private static final String KEY_ISBT_128 = "code128_isbt_128";
    private static final String KEY_MIN = "code128_min";
    private static final String KEY_MAX = "code128_max";

    private static final int MAX_LENGTH = 127;
    private static final int MIN_LENGTH = 1;

    //private SwitchPreference mGs128Pref;
    private SwitchPreference mIsbt128Pref;

    //add by yangzhichao for t0581 20201009 start
    private SwitchPreference m128ReversePref;
    //add by yangzhichao for t0581 20201009 end

    private Preference mMaxPref;
    private Preference mMinPref;

    //add by yangzhichao for t0581 20201009 start
    private static final String KEY_128_REVERSE = "code_128";
    //add by yangzhichao for t0581 20201009 end

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //add by yangzhichao for t0581 20201009 start
        int code128EnableValue = getDecoderTagValue(XCScanner.TAG_CODE128_INVERSE);

        m128ReversePref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_128_REVERSE, getString(R.string.text_128_reverse_enable), XCScanner.TAG_CODE128_INVERSE);
        m128ReversePref.setChecked((code128EnableValue & 1) == 1 && code128EnableValue != 0);
        m128ReversePref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code128Setting.this).edit();
                editor.putInt(KEY_128_REVERSE, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE128_INVERSE,
                    checked ?
                    getDecoderTagValue(XCScanner.TAG_CODE128_INVERSE) | (0x01) :
                    getDecoderTagValue(XCScanner.TAG_CODE128_INVERSE) & (~(0x01)));
                return true;
            }
        });
        //add by yangzhichao for t0581 20201009 end

        mIsbt128Pref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_ISBT_128, getString(R.string.text_isbt_128), XCScanner.TAG_C128_ISBT_ENABLED/*ISBT_128_PARM*/);
        //mIsbt128Pref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_ISBT_128, -1) == 1);
        Log.d(TAG, "mIsbt128Pref = " + getDecoderTagValue(XCScanner.TAG_C128_ISBT_ENABLED));
        mIsbt128Pref.setChecked(getDecoderTagValue(XCScanner.TAG_C128_ISBT_ENABLED) == 1);

        mIsbt128Pref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code128Setting.this).edit();
                editor.putInt(KEY_ISBT_128, checked ? 1 : 0);
                editor.commit();
                //setParameter(ISBT_128_PARM, checked ? 1 : 0);
                setDecoderTag(XCScanner.TAG_C128_ISBT_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_CODE128_MAX_LENGTH/*MAX_PARM*/);
        int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue2 = getDecoderTagValue(XCScanner.TAG_CODE128_MAX_LENGTH);
        Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Code128Setting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE128_MAX_LENGTH/*MAX_PARM*/);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_CODE128_MIN_LENGTH/*MIN_PARM*/);
        int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue2 = getDecoderTagValue(XCScanner.TAG_CODE128_MIN_LENGTH);
        Log.d(TAG, "minValue = " + minValue + "minValue2 = " + minValue2);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Code128Setting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE128_MIN_LENGTH/*MIN_PARM*/);
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
