package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;


public class Code39Setting extends BaseSetting {

    private static final String TAG = "Code39Setting";

    private static final String KEY_TRANSMIT = "code39_transmit";
    private static final String KEY_APPEND = "code39_append";
    private static final String KEY_VERIFY = "code39_verify";
    private static final String KEY_FULL_ASC = "code39_full_asc";
    private static final String KEY_CONVERT = "code39_convert_to_code32";

    private static final String KEY_MIN = "code39_min";
    private static final String KEY_MAX = "code39_max";

    private static final int MAX_LENGTH = 127;
    private static final int MIN_LENGTH = 4;
    private static final int DFAULT_MIN_LENGTH = 2;

    private SwitchPreference mTransmitPref;
    private SwitchPreference mAppend;
    private SwitchPreference mVerifyPref;
    private SwitchPreference mFullAscPref;
    private SwitchPreference mConvertPref;
    private Preference mMaxPref;
    private Preference mMinPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mTransmitPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT, getString(R.string.transmit_check_digit), XCScanner.TAG_CODE39_START_STOP_TRANSMIT);
        //mTransmitPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT, -1) == 1);
        Log.d(TAG, "mTransmitPref = " + getDecoderTagValue(XCScanner.TAG_CODE39_START_STOP_TRANSMIT));
        mTransmitPref.setChecked(getDecoderTagValue(XCScanner.TAG_CODE39_START_STOP_TRANSMIT) == 1);

        mTransmitPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code39Setting.this).edit();
                editor.putInt(KEY_TRANSMIT, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE39_START_STOP_TRANSMIT, checked ? 1 : 0);
                return true;
            }
        });

        mAppend = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_APPEND, getString(R.string.code39_expand), XCScanner.TAG_CODE39_APPEND_ENABLED);
        //mTransmitPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT, -1) == 1);
        Log.d(TAG, "mAppend = " + getDecoderTagValue(XCScanner.TAG_CODE39_APPEND_ENABLED));
        mAppend.setChecked(getDecoderTagValue(XCScanner.TAG_CODE39_APPEND_ENABLED) == 1);

        mAppend.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code39Setting.this).edit();
                editor.putInt(KEY_APPEND, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE39_APPEND_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mVerifyPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_VERIFY, getString(R.string.verify_check_digit), XCScanner.TAG_CODE39_CHECK_DIGIT_MODE/*VERIFY_PARM*/);
        //mVerifyPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_VERIFY, -1) == 1);
        Log.d(TAG, "mVerifyPref = " + getDecoderTagValue(XCScanner.TAG_CODE39_CHECK_DIGIT_MODE));
        mVerifyPref.setChecked(getDecoderTagValue(XCScanner.TAG_CODE39_CHECK_DIGIT_MODE) == 1);

        mVerifyPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code39Setting.this).edit();
                editor.putInt(KEY_VERIFY, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE39_CHECK_DIGIT_MODE, checked ? 1 : 0);
                return true;
            }
        });

        mFullAscPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_FULL_ASC, getString(R.string.code39_full_asc), XCScanner.TAG_CODE39_FULL_ASCII_ENABLED/*FULL_ASC_PARM*/);
        //mFullAscPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_FULL_ASC, -1) == 1);
        Log.d(TAG, "mFullAscPref = " + getDecoderTagValue(XCScanner.TAG_CODE39_FULL_ASCII_ENABLED));
        mFullAscPref.setChecked(getDecoderTagValue(XCScanner.TAG_CODE39_FULL_ASCII_ENABLED) == 1);

        mFullAscPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code39Setting.this).edit();
                editor.putInt(KEY_FULL_ASC, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE39_FULL_ASCII_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mConvertPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_CONVERT, getString(R.string.code39_base32), XCScanner.TAG_CODE39_BASE32_ENABLED/*CONVERT_PARM*/);
        //mConvertPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_CONVERT, -1) == 1);
        Log.d(TAG, "mConvertPref = " + getDecoderTagValue(XCScanner.TAG_CODE39_BASE32_ENABLED));
        mConvertPref.setChecked(getDecoderTagValue(XCScanner.TAG_CODE39_BASE32_ENABLED) == 1);

        mConvertPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Code39Setting.this).edit();
                editor.putInt(KEY_CONVERT, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_CODE39_BASE32_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_CODE39_MAX_LENGTH/*MAX_PARM*/);
        int maxValue = BaseSetting.getSharedPreferences(Code39Setting.this).getInt(KEY_MAX, -1);
        int maxValue2 = getDecoderTagValue(XCScanner.TAG_CODE39_MAX_LENGTH);
        Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Code39Setting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE39_MAX_LENGTH/*MAX_PARM*/);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_CODE39_MIN_LENGTH/*MIN_PARM*/);
        int minValue = BaseSetting.getSharedPreferences(Code39Setting.this).getInt(KEY_MIN, -1);
        int minValue2 = getDecoderTagValue(XCScanner.TAG_CODE39_MIN_LENGTH);
        Log.d(TAG, "minValue = " + minValue + ", minValue2 = " + minValue2);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(DFAULT_MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Code39Setting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_CODE39_MIN_LENGTH/*MIN_PARM*/);
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
