package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.E3Util;
import com.xcheng.scannere3.XCScanner;

public class CodeBlockFSetting extends BaseSetting {

    private static final String TAG = "CodeBlockFSetting";

    private static final int MAX_PARM = XCScanner.TAG_CODABLOCK_F_MAX_LENGTH;
    private static final int MIN_PARM = XCScanner.TAG_CODABLOCK_F_MIN_LENGTH;

    private static final String KEY_MIN = "codeblock_f_min";
    private static final String KEY_MAX = "codeblock_f_max";

    private static final int MAX_LENGTH = 55;
    private static final int MIN_LENGTH = 0;
    private static final int DFAULT_MIN_LENGTH = 4;

    private Preference mMaxPref;
    private Preference mMinPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //modify by yangzhichao for t0581 20201016 start
        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, MAX_PARM);
        //int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue = getDecoderTagValue(MAX_PARM);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(CodeBlockFSetting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, MAX_PARM);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, MIN_PARM);
        //int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue = getDecoderTagValue(MIN_PARM);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(DFAULT_MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(CodeBlockFSetting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, MIN_PARM);
                return true;
            }
        });
        //modify by yangzhichao for t0581 20201015 start
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
