package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;

//add by yangzhichao for t0581 20201017 start
import com.xcheng.scannere3.XCScanner;
//add by yangzhichao for t0581 20201017 end

public class Matrix2of5Setting extends BaseSetting {

    private static final String TAG = "Matrix2of5Setting";

    private static final int TRANSMIT_PARM = 623;
    private static final int CHECK_PARM = XCScanner.TAG_M25_CHECK_DIGIT_MODE;
    private static final int REDUNDANCY_PARM = 621;
    private static final int MAX_PARM = XCScanner.TAG_M25_MAX_LENGTH;
    private static final int MIN_PARM = XCScanner.TAG_M25_MIN_LENGTH;

    private static final String KEY_TRANSMIT = "matrix2of5_transmit";
    private static final String KEY_CHECK = "matrix2of5_check_digit";
    private static final String KEY_REDUNDANCY = "matrix2of5_redundancy";
    private static final String KEY_MIN = "matrix2of5_min";
    private static final String KEY_MAX = "matrix2of5_max";

    private static final int MAX_LENGTH = 127;
    private static final int MIN_LENGTH = 6;
    private static final int DFAULT_MIN_LENGTH = 14;

    private SwitchPreference mTransmitPref;
    private SwitchPreference mCheckPref;
    private SwitchPreference mRedundancyPref;
    private Preference mMaxPref;
    private Preference mMinPref;
    //modify by yangzhichao for t0581 20201017 start

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //modify by yangzhichao for t0581 20201017 start
        /*
        mCheckPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_CHECK, getString(R.string.check_digit), CHECK_PARM);
        mCheckPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_CHECK, -1) == 1);
        mCheckPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Matrix2of5Setting.this).edit();
                editor.putInt(KEY_CHECK, checked ? 1 : 0);
                editor.commit();
                setParameter(CHECK_PARM, checked ? 1 : 0);
                return true;
            }
        });
        */
        mTransmitPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT, getString(R.string.transmit_check_digit), TRANSMIT_PARM);
        //mTransmitPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT, -1) == 1);
        mTransmitPref.setChecked(getDecoderTagValue(TRANSMIT_PARM) == 1);
        mTransmitPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Matrix2of5Setting.this).edit();
                editor.putInt(KEY_TRANSMIT, checked ? 1 : 0);
                editor.commit();
                setParameter(TRANSMIT_PARM, checked ? 1 : 0);
                return true;
            }
        });
        /*
        mRedundancyPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_REDUNDANCY, getString(R.string.check_digit), REDUNDANCY_PARM);
        mRedundancyPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_REDUNDANCY, -1) == 1);
        mRedundancyPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Matrix2of5Setting.this).edit();
                editor.putInt(KEY_REDUNDANCY, checked ? 1 : 0);
                editor.commit();
                setParameter(REDUNDANCY_PARM, checked ? 1 : 0);
                return true;
            }
        });
        */
        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, MAX_PARM);
        //int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue = getDecoderTagValue(MAX_PARM);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Matrix2of5Setting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, MAX_PARM);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, MIN_PARM);
        //int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue = getDecoderTagValue(MIN_PARM);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(DFAULT_MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Matrix2of5Setting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, MIN_PARM);
                return true;
            }
        });
        //modify by yangzhichao for t0581 20201017 end
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
