package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;


public class QrCodeSetting extends BaseSetting {

    private static final String TAG = "QrCodeSetting";

    private static final String KEY_MIN = "qrcode_min";
    private static final String KEY_MAX = "qrcode_max";

    private static final int MAX_LENGTH = 7089;
    private static final int MIN_LENGTH = 0;

    private Preference mMaxPref;
    private Preference mMinPref;

    private static final String KEY_QR_REVERSE = "qr_reverse";
    private static final String KEY_MQR = "qr_mqr";
    private static final String KEY_MQR_REVERSE = "qr_mqr_reverse";
    private static final String KEY_MIRROR = "qr_mirror";

    private SwitchPreference mQrReversePref;
    private SwitchPreference mMQrPref;
    private SwitchPreference mMQrReversePref;
    private SwitchPreference mMirrorPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        int qrEnableValue = getDecoderTagValue(XCScanner.TAG_QR_ENABLED);

        mQrReversePref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_QR_REVERSE, getString(R.string.text_qr_reverse_enable), XCScanner.TAG_QR_ENABLED);
        mQrReversePref.setChecked((qrEnableValue & (0x01<<1)) == (0x01<<1) && qrEnableValue != 0);

        mQrReversePref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(QrCodeSetting.this).edit();
                editor.putInt(KEY_QR_REVERSE, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_QR_ENABLED,
                    checked ?
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) | (0x01<<1) :
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) & (~(0x01<<1)));
                return true;
            }
        });

        mMQrPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_MQR, getString(R.string.text_mqr_enable), XCScanner.TAG_QR_ENABLED);
        mMQrPref.setChecked((qrEnableValue & (0x01<<2)) == (0x01<<2) && qrEnableValue != 0);

        mMQrPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(QrCodeSetting.this).edit();
                editor.putInt(KEY_MQR, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_QR_ENABLED,
                    checked ?
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) | (0x01<<2) :
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) & (~(0x01<<2)));
                return true;
            }
        });

        mMQrReversePref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_MQR_REVERSE, getString(R.string.text_mqr_reverse_enable), XCScanner.TAG_QR_ENABLED);
        mMQrReversePref.setChecked((qrEnableValue & (0x01<<3)) == (0x01<<3) && qrEnableValue != 0);

        mMQrReversePref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(QrCodeSetting.this).edit();
                editor.putInt(KEY_MQR_REVERSE, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_QR_ENABLED,
                    checked ?
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) | (0x01<<3) :
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) & (~(0x01<<3)));
                return true;
            }
        });

        mMirrorPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_MIRROR, getString(R.string.text_mirror_enable), XCScanner.TAG_QR_ENABLED);
        mMirrorPref.setChecked((qrEnableValue & (0x01<<4)) == (0x01<<4) && qrEnableValue != 0);

        mMirrorPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(QrCodeSetting.this).edit();
                editor.putInt(KEY_MIRROR, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_QR_ENABLED,
                    checked ?
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) | (0x01<<4) :
                    getDecoderTagValue(XCScanner.TAG_QR_ENABLED) & (~(0x01<<4)));
                return true;
            }
        });

        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_QR_MAX_LENGTH);
        //int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue = getDecoderTagValue(XCScanner.TAG_QR_MAX_LENGTH);
        //Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(QrCodeSetting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_QR_MAX_LENGTH);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_QR_MIN_LENGTH);
        //int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue = getDecoderTagValue(XCScanner.TAG_QR_MIN_LENGTH);
        //Log.d(TAG, "minValue = " + minValue + "minValue2 = " + minValue2);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(QrCodeSetting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_QR_MIN_LENGTH);
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
