package com.xcheng.scannere3.scantest;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

import com.xcheng.scannere3.*;

public class ScanContinueActivity extends Activity {
    private static final String TAG = "ScanContinueActivity";

    public static final int MSG_RESULT_UPDATE = 0x01;
    
    private static final Object sLock = new Object();
    
    private double mQRStartSpace;
    private double mQRCurrentStartSpace;
    private double mQRCurrentStartSpaceBackup;
    private double mQRCallbackSpace;

    private boolean mScanLoopStart = false;

    private TextView mShowCount;
    private TextView mShowAverageTime;
    private TextView mShowSingleTime;
    private TextView mShowTimesPerMinute;
    private TextView mShowFormat;
    private TextView mShowResult;

    private String mCodeFormat;
    private String mCodeResult;

    private int mScanMode = 0;

    private boolean mIsNeedResetScanMode = false;

    private List<ScanDataItem> mScanResultList;

    private class ScanDataItem {
        private String symbology;
        private String barcode;
        private long id; // record of ID
        private long tmStart; // time stamp of start decode
        private long tmDone;  // time stamp of decode done
        private int state; // 0 - Inited, 1 - Finished

        public ScanDataItem (long id, String symbology, String barcode) {
            super();
            this.id = id;
            this.symbology = symbology;
            this.barcode = barcode;
        }

        public ScanDataItem (String symbology, String barcode) {
            super();
            this.id = -1;
            this.symbology = symbology;
            this.barcode = barcode;
        }

        public ScanDataItem (long id) {
            super();
            this.id = id;
        }

        public long getId() {
            return this.id;
        }

        public long getTimeStart() {
            return this.tmStart;
        }

        public long getTimeDone() {
            return this.tmDone;
        }

        public long getTimeElapse() {
            return (this.tmDone - this.tmStart);
        }

        public String getSymbology() {
            return this.symbology;
        }

        public String getBarcode() {
            return this.barcode;
        }

        public void setId(long id) {
            this.id = id;
        }

        public void setSymbology(String sym) {
            this.symbology = sym;
        }

        public void setBarcode(String barcode) {
            this.barcode = barcode;
        }

        public void setTimeStart(long tm) {
            this.tmStart = tm;
            this.state = 0;
        }

        public void setTimeDone(long tm) {
            this.tmDone = tm;
            this.state = 1;
        }

        public boolean isCompleted() {
            return (this.state == 1);
        }
    }

    @SuppressLint("HandlerLeak")
    private final Handler mHandler = new Handler() {
        @SuppressLint("SetTextI18n")
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            switch(msg.what) {
                case MSG_RESULT_UPDATE:
//                    Log.d(TAG, "handleMessage: MSG_RESULT_UPDATE");

                    String symbology = null;
                    String barcode = null;

                    // Show latest record
                    if (mScanResultList.size() > 0) {
                        int tailId = mScanResultList.size() - 1;
                        ScanDataItem item = mScanResultList.get(tailId);

                        symbology = item.getSymbology();
                        barcode = item.getBarcode();

                        mShowFormat.setText(symbology);
                        mShowResult.setText(barcode);
                    }

                    if (mScanResultList.size() > 0) {
                        int tailId = mScanResultList.size() - 1;
                        ScanDataItem firstItem = mScanResultList.get(0);
                        ScanDataItem lastItem = mScanResultList.get(tailId);
                        long startTime = firstItem.getTimeStart();
                        long endTime = lastItem.getTimeDone();

                        int averageTimes = (int) ((endTime - startTime) / mScanResultList.size());
                        int singleTimes = (int) lastItem.getTimeElapse();
                        int cpm = (60*1000)/averageTimes;

                        mShowCount.setText(mScanResultList.size() + "");
                        mShowAverageTime.setText(averageTimes + "");
                        mShowTimesPerMinute.setText(cpm + "");
                        mShowSingleTime.setText(singleTimes + "");
                    }

                    if (mScanLoopStart) {
                        // Insert new record for next result
                        //Log.d(TAG, "mScanLoopStart Start scan again");

                        int idx = mScanResultList.size();
                        ScanDataItem newitem = new ScanDataItem((long)idx);
                        newitem.setTimeStart(System.currentTimeMillis());
                        mScanResultList.add(idx, newitem);
                    }
                    break;
                    
                default:
                    break;
            }
        }
    };

    private final XCScanner.Result mScanContinuxActivityResultListener = new XCScanner.Result(){
        @Override
        public void scanBeep() {
            //Log.d(TAG, "mScanContinuxActivityResultListener.scanBeep");
        }

        @Override
        public void scanStart() {
            //Log.d(TAG, "mScanContinuxActivityResultListener.scanStart");
        }

        @Override
        public void scanResult(String symbology,String barcode) {
            //Log.d(TAG, "mScanContinuxActivityResultListener.scanResult");
            //modify by yangzhichao for t0581 20201028 start
            if(mScanResultList.size() > 0){
                int tailId = mScanResultList.size() - 1;
                ScanDataItem scanData = mScanResultList.get(tailId);
                if (scanData != null) {
                    if (!scanData.isCompleted()) {
                        scanData.setTimeDone(System.currentTimeMillis());
                        scanData.setSymbology(symbology);
                        scanData.setBarcode(barcode);
                        mScanResultList.set(tailId, scanData);
                    }
                }
            }
            //modify by yangzhichao for t0581 20201028 end
            // Can not update UI here, send message to handler to update UI
            mHandler.sendEmptyMessage(MSG_RESULT_UPDATE);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.d(TAG, "onCreate+");
        super.onCreate(savedInstanceState);

        this.setContentView(R.layout.activity_scan_continue);
        initView();

        this.mScanResultList = new ArrayList<ScanDataItem>();

        Log.d(TAG, "onCreate-");
    }

    private void initView() {
        this.mShowCount = (TextView)this.findViewById(R.id.show_count);
        this.mShowAverageTime = (TextView)this.findViewById(R.id.show_average_time);
        this.mShowTimesPerMinute = (TextView)this.findViewById(R.id.show_times_per_minute);
        this.mShowFormat = (TextView)this.findViewById(R.id.show_format);
        this.mShowResult = (TextView)this.findViewById(R.id.show_result);
        this.mShowSingleTime = (TextView)this.findViewById(R.id.show_single_time);
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume+");
        
        super.onResume();

        // Notify service: enter scan test activity
        Intent intent = new Intent(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_ENTER);
        sendBroadcast(intent);

        ScanTestService.setupScanServiceResultListener(mScanContinuxActivityResultListener);
        
        Log.d(TAG, "onResume-");
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause+");
        super.onPause();

        // Make sure close scan
        if (E3Util.IsDecoding()) {
            mScanLoopStart = false;
            E3Util.stopDecode();
        }

        // Make sure reset scan mode
        if (mIsNeedResetScanMode) {
            E3Util.setScanMode(mScanMode);
            mIsNeedResetScanMode = false;
        }

        this.mScanResultList.clear();

        // Notify service: leave scan test activity
        Intent intent = new Intent(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_LEAVE);
        sendBroadcast(intent);

        ScanTestService.setupScanServiceResultListener(null);

        Log.d(TAG, "onPause-");
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy+");
        
        super.onDestroy();

        // Make sure close scan
        if (E3Util.IsDecoding()) {
            mScanLoopStart = false;
            E3Util.stopDecode();
        }

        // Make sure reset scan mode
        if (mIsNeedResetScanMode) {
            E3Util.setScanMode(mScanMode);
            mIsNeedResetScanMode = false;
        }

        this.mScanResultList.clear();
        
        ScanTestService.setupScanServiceResultListener(null);

        Log.d(TAG, "onDestroy-");
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        synchronized (sLock) {
            Log.d(TAG, "onKeyDown keyCode = " + keyCode + ", event = " + event);
            if (keyCode == KeyEvent.KEYCODE_FUNCTION
                    || keyCode == KeyEvent.KEYCODE_F11
                    || keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN
                    || keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {

                if (event.getRepeatCount() != 0) {
                    // Ignore long press down
                    return true;
                }

                if (!mScanLoopStart) {
                    mScanMode = E3Util.getScanMode();
                    if (mScanMode != XCScanner.WORKMODE_CONTINUOUS_SIMPLE) {
                        mIsNeedResetScanMode = true;
                        E3Util.setScanMode(XCScanner.WORKMODE_CONTINUOUS_SIMPLE);
                    }

                    // Start at first press down
                    mScanLoopStart = true;
                    Log.d(TAG, "mScanLoopStart " + mScanLoopStart);

                    // Clear all old record
                    this.mScanResultList.clear();

                    // First scan record
                    int idx = this.mScanResultList.size();
                    ScanDataItem item = new ScanDataItem((long)idx);
                    item.setTimeStart(System.currentTimeMillis());
                    mScanResultList.add(idx, item);

                    // Start scan
                    E3Util.doDecode();
                } else {
                    // Stop at second press down
                    mScanLoopStart = false;
                    Log.d(TAG, "mScanLoopStart " + mScanLoopStart);

                    // Stop scan
                    E3Util.stopDecode();

                    if (mIsNeedResetScanMode) {
                        E3Util.setScanMode(mScanMode);
                        mIsNeedResetScanMode = false;
                    }
                }
                
                return true;
            }

            return super.onKeyDown(keyCode, event);
        }
    }
}

